searchNodes=[{"ref":"Jibe.html","title":"Jibe","type":"module","doc":"Tools for checking if an arbitrarily nested map/list matches a particular pattern. The genesis of this was as a helper in unit tests to check if JSON was being generated as expected. This does not require any particular JSON library, so if you want to use it with JSON you&#39;ll need to decode it with something like Poison.decode/1 first. Note that these decode functions will usually build maps with strings for keys, as opposed to atoms, so set up your pattern accordingly."},{"ref":"Jibe.html#match?/2","title":"Jibe.match?/2","type":"function","doc":"Given a pattern and an actual map/list, determine if the pattern is a subset of the actual. The pattern is &quot;forgiving&quot; in the sense that extra keys or list elements in the actual are OK. However, every key and/or element in the pattern must appear in the actual. Anything missing in the actual is a failure. List order is significant by default. Examples # Simple equality tests. iex&gt; Jibe.match?(%{}, %{}) true iex&gt; Jibe.match?([], []) true # Hash without nested values. Order of keys is not important, # since hash keys aren&#39;t ordered in the first place. iex&gt; Jibe.match?(%{&quot;foo&quot; =&gt; &quot;bar&quot;, &quot;a&quot; =&gt; 123}, %{&quot;a&quot; =&gt; 123, &quot;foo&quot; =&gt; &quot;bar&quot;}) true # The keys are good, but the values don&#39;t match. iex&gt; Jibe.match?(%{foo: :bar}, %{foo: :not_bar}) false # Fails because not all expected hash elements are found. iex&gt; Jibe.match?(%{&quot;foo&quot; =&gt; &quot;bar&quot;, &quot;a&quot; =&gt; 123}, %{&quot;a&quot; =&gt; 123}) false # Also works in nested maps. (`y: 2` is missing from the `actual`) iex&gt; Jibe.match?(%{foo: :bar, a: %{x: 1, y: 2}}, %{a: %{x: 1}, foo: :bar}) false # Extra keys in the actual don&#39;t matter. iex&gt; Jibe.match?(%{&quot;foo&quot; =&gt; &quot;bar&quot;}, %{&quot;a&quot; =&gt; 123, &quot;foo&quot; =&gt; &quot;bar&quot;}) true iex&gt; Jibe.match?(%{foo: :bar, a: %{x: 1}}, %{a: %{x: 1, y: 2}, foo: :bar}) true # Simple list comparison. iex&gt; Jibe.match?([1,2,3], [1,2,3]) true # Extra list elements in the actual are OK. iex&gt; Jibe.match?([1,2,3], [1,2,3,4]) true iex&gt; Jibe.match?([1,2, [3,4]], [1,2, [3,4,5]]) true # The position of extra list elements does not matter. iex&gt; Jibe.match?([2,4], [1,2,3,4,5]) true # Missing list elements in the actual are a failure. iex&gt; Jibe.match?([1,2,3], [1,2]) false iex&gt; Jibe.match?([1,2, [3,4]], [1,2, [3]]) false # Mix and matching maps and lists. iex&gt; Jibe.match?([1, %{foo: :bar}], [0, 1, 2, %{foo: :bar}, 3]) true # Nested data where both sides are the same. iex&gt; pattern = [%{&quot;a&quot; =&gt; 1, &quot;b&quot; =&gt; [2, 3, %{&quot;x&quot; =&gt; [4,5]}]}, 9, 10] iex&gt; actual = [%{&quot;a&quot; =&gt; 1, &quot;b&quot; =&gt; [2, 3, %{&quot;x&quot; =&gt; [4,5]}]}, 9, 10] iex&gt; Jibe.match?(pattern, actual) true # Nested data, actual is missing a list element. iex&gt; pattern = [%{&quot;a&quot; =&gt; 1, &quot;b&quot; =&gt; [2, 3, %{&quot;x&quot; =&gt; [4,5]}]}, 9, 10] iex&gt; actual = [%{&quot;a&quot; =&gt; 1, &quot;b&quot; =&gt; [2, 3, %{&quot;x&quot; =&gt; [4]}]}, 9, 10] iex&gt; Jibe.match?(pattern, actual) false # Nested data, actual has an extra list element, which is fine. iex&gt; pattern = [%{&quot;a&quot; =&gt; 1, &quot;b&quot; =&gt; [2, 3, %{&quot;x&quot; =&gt; [4]}]}, 9, 10] iex&gt; actual = [%{&quot;a&quot; =&gt; 1, &quot;b&quot; =&gt; [2, 3, %{&quot;x&quot; =&gt; [4,5]}]}, 9, 10] iex&gt; Jibe.match?(pattern, actual) true # Nested data, actual map inside of a list has an extra element. iex&gt; pattern = [%{&quot;a&quot; =&gt; 1}, %{&quot;a&quot; =&gt; 2}] iex&gt; actual = [%{&quot;a&quot; =&gt; 1, &quot;x&quot; =&gt; 9}, %{&quot;y&quot; =&gt; 9, &quot;a&quot; =&gt; 2}] iex&gt; Jibe.match?(pattern, actual) true # Finding the matching map within a list of maps iex&gt; pattern = [%{foo: :bar}] iex&gt; actual = [%{foo: :x}, %{foo: :bar}, %{foo: :y}] iex&gt; Jibe.match?(pattern, actual) true Tuples They&#39;re converted to lists insternally, so the same rules apply. iex&gt; pattern = {1, 2} iex&gt; actual = {1, 2} iex&gt; Jibe.match?(pattern, actual) true iex&gt; pattern = {1, 2} iex&gt; actual = {1, 2, 3} iex&gt; Jibe.match?(pattern, actual) true iex&gt; pattern = {1, 2} iex&gt; actual = {1} iex&gt; Jibe.match?(pattern, actual) false iex&gt; pattern = [{1, 2}, {3, 4}] iex&gt; actual = [{1, 2, 3}, {3, 5, 4}] iex&gt; Jibe.match?(pattern, actual) true Unsorted lists This is a performance hit on your tests. Avoid large lists if you&#39;re especially worried about speed. iex&gt; Jibe.match?({:unsorted, [1, 2]}, [2, 1]) true Wildcard matches # &quot;Something&quot; needs to be there, but we don&#39;t care what it is. iex&gt; Jibe.match?([1, :wildcard, 3], [1, 999, 3]) true # :wildcard still needs the key to be present in a map. iex&gt; Jibe.match?(%{&quot;foo&quot; =&gt; :wildcard}, %{&quot;x&quot; =&gt; &quot;bar&quot;}) false # :wildcard will match a nil value as long as the key is there. iex&gt; Jibe.match?(%{&quot;foo&quot; =&gt; :wildcard}, %{&quot;foo&quot; =&gt; nil}) true DateTime matching iex&gt; {:ok, d1, _} = DateTime.from_iso8601(&quot;2018-01-01T12:00:00Z&quot;) iex&gt; {:ok, d2, _} = DateTime.from_iso8601(&quot;2018-01-01T12:00:00.000000Z&quot;) iex&gt; Jibe.match?([d1], [d2]) true iex&gt; {:ok, d1, _} = DateTime.from_iso8601(&quot;2018-01-01T12:00:00Z&quot;) iex&gt; {:ok, d2, _} = DateTime.from_iso8601(&quot;2018-01-01T12:00:00.000000Z&quot;) iex&gt; Jibe.match?(%{d: d1}, %{d: d2}) true iex&gt; {:ok, d1, _} = DateTime.from_iso8601(&quot;2018-01-01T12:00:00Z&quot;) iex&gt; {:ok, d2, _} = DateTime.from_iso8601(&quot;2000-01-01T12:00:00Z&quot;) iex&gt; Jibe.match?([d1], [d2]) false Decimal matching # Decimal values require a special comparison iex&gt; Jibe.match?([Decimal.new(2)], [Decimal.from_float(2.0)]) true iex&gt; Jibe.match?([Decimal.new(4)], [Decimal.from_float(4.5)]) false"},{"ref":"readme.html","title":"Jibe","type":"extras","doc":"Jibe An elixir library for comparing an arbitrarily nested map/list against a pattern. This is intended as a helper in unit tests, but there&#39;s nothing forcing that to be the case. It&#39;s just a simple set of elixir functions that could be called from anywhere. Having said that, I use it in a lot of tests and pretty much only in tests."},{"ref":"readme.html#api-docs","title":"Jibe - API Docs","type":"extras","doc":"API docs are here"},{"ref":"readme.html#example","title":"Jibe - Example","type":"extras","doc":"pattern = [:red, :green] # Partial matches are fine. Jibe.match?(pattern, [:blue, :red, :purple, :green]) true # Missing elements are no good though. # In this case :red was expected and not found. Jibe.match?(pattern, [:green]) false # Order matters by deault. Jibe.match?(pattern, [:green, :red]) false # Or you don&#39;t care about order. Jibe.match?({:unsorted, pattern}, [:green, :red]) true The pattern can be an arbitraily nested structure of maps and lists. See the docs for examples using maps, dates, wildcards, and more."},{"ref":"readme.html#what-does-this-look-like-in-a-typical-unit-test","title":"Jibe - What does this look like in a typical unit test?","type":"extras","doc":"I originally made this to help test API results in Phoenix apps. If you don&#39;t use Phoenix just imagine that we have a thing that is spewing out JSON, and we&#39;d like to check some or all of our fields. JSON deserialization is external to Jibe. Use Poison or Jason or whatever you want. # Again, I&#39;m assuming &quot;Phoenix&quot; here, but Jibe doesn&#39;t care. defmodule MyAppWeb.UsersControllerTest do use MyApp.ConnCase import MyApp.Factory # using `ex_machina` in this case. setup do fred = insert(:user, first_name: &quot;Fred&quot;, last_name: &quot;Flintstone&quot;) barney = insert(:user, first_name: &quot;Barney&quot;, last_name: &quot;Rubble&quot;) wilma = insert(:user, first_name: &quot;Wilma&quot;, last_name: &quot;Flintstone&quot;) conn = build_conn() # |&gt; whatever else you need to do to set up your conn (login, etc.) {:ok, conn: conn, users: [fred, barney, wilma]} end test &quot;index of all users&quot;, %{conn: conn, users: [fred, barney, wilma]} do data = conn |&gt; get(users_path(conn, :index)) |&gt; json_response(200) # JSON is deserialized into Elixir maps/lists here # Usually I&#39;ll check the shape of one of the results, and then just look for IDs # or something smaller than &quot;the entire record&quot; for the rest. This works because # Jibe assumes that your pattern is a subset of the actual data. # If I know and expect to enforce a particular sort order. pattern = [ %{ &quot;id&quot; =&gt; fred.id, &quot;first_name&quot; =&gt; &quot;Fred&quot;, &quot;last_name&quot; =&gt; &quot;Flintstone&quot; }, %{ &quot;id&quot; =&gt; barney.id }, %{ &quot;id&quot; =&gt; wilma.id } ] assert Jibe.match?(pattern, data) # If instead I don&#39;t care about the sort order. # This is slower to execute, but in a small unit test it barely matters. pattern = {:unsorted, [ %{ &quot;id&quot; =&gt; fred.id, &quot;first_name&quot; =&gt; &quot;Fred&quot;, &quot;last_name&quot; =&gt; &quot;Flintstone&quot; }, %{ &quot;id&quot; =&gt; barney.id}, %{ &quot;id&quot; =&gt; wilma.id} ]} assert Jibe.match?(pattern, data) # Or maybe I only care that &quot;Barney&quot; showed up somewhere in the results, not caring at all about # the rest of the list. pattern = [ %{&quot;id&quot; =&gt; barney.id} ] assert Jibe.match?(pattern, data) end"},{"ref":"readme.html#installation","title":"Jibe - Installation","type":"extras","doc":"The package can be installed by adding jibe to your list of dependencies in mix.exs: def deps do [ {:jibe, &quot;~&gt; 0.2.1&quot;} ] end"}]